@@ -1,46 +1,80 @@
-//! Build and publish crates with pyo3, rust-cpython and cffi bindings as well as rust binaries
+//! Build and publish crates with pyo3, cffi and uniffi bindings as well as rust binaries
 //! as python packages. This file contains the CLI and keyring integration.
 //!
 //! Run with --help for usage information
 
-use anyhow::{bail, Context, Result};
+use anyhow::{Context, Result, bail};
+use cargo_options::heading;
 #[cfg(feature = "zig")]
 use cargo_zigbuild::Zig;
 #[cfg(feature = "cli-completion")]
 use clap::CommandFactory;
 use clap::{Parser, Subcommand};
-#[cfg(feature = "scaffolding")]
-use maturin::{ci::GenerateCI, init_project, new_project, GenerateProjectOptions};
+use ignore::overrides::Override;
 use maturin::{
-    develop, write_dist_info, BridgeModel, BuildOptions, CargoOptions, DevelopOptions, PathWriter,
-    PlatformTag, PythonInterpreter, Target,
+    BridgeModel, BuildOptions, CargoOptions, DevelopOptions, PathWriter, PythonInterpreter, Target,
+    TargetTriple, VirtualWriter, develop, find_path_deps, write_dist_info,
 };
+#[cfg(feature = "schemars")]
+use maturin::{GenerateJsonSchemaOptions, generate_json_schema};
+#[cfg(feature = "scaffolding")]
+use maturin::{GenerateProjectOptions, ci::GenerateCI, init_project, new_project};
 #[cfg(feature = "upload")]
-use maturin::{upload_ui, PublishOpt};
+use maturin::{PublishOpt, upload_ui};
 use std::env;
 use std::path::PathBuf;
-use tracing::debug;
+use std::str::FromStr;
+use tracing::{debug, instrument};
+use tracing_subscriber::filter::Directive;
+use tracing_subscriber::{EnvFilter, Layer, layer::SubscriberExt, util::SubscriberInitExt};
 
 #[derive(Debug, Parser)]
 #[command(
     version,
     name = env!("CARGO_PKG_NAME"),
     display_order = 1,
-    after_help = "Visit https://maturin.rs to learn more about maturin."
+    after_help = "Visit https://maturin.rs to learn more about maturin.",
+    styles = cargo_options::styles(),
 )]
-#[cfg_attr(feature = "cargo-clippy", allow(clippy::large_enum_variant))]
-/// Build and publish crates with pyo3, rust-cpython and cffi bindings as well
+/// Build and publish crates with pyo3, cffi and uniffi bindings as well
 /// as rust binaries as python packages
-enum Opt {
+struct Opt {
+    /// Use verbose output.
+    ///
+    /// * Default: Show build information and `cargo build` output.
+    /// * `-v`: Use `cargo build -v`.
+    /// * `-vv`: Show debug logging and use `cargo build -vv`.
+    /// * `-vvv`: Show trace logging.
+    ///
+    /// You can configure fine-grained logging using the `RUST_LOG` environment variable.
+    /// (<https://docs.rs/tracing-subscriber/latest/tracing_subscriber/filter/struct.EnvFilter.html#directives>)
+    #[arg(global = true, action = clap::ArgAction::Count, long, short)]
+    verbose: u8,
+    #[command(subcommand)]
+    command: Command,
+}
+
+#[derive(Debug, Parser)]
+/// Build and publish crates with pyo3, cffi and uniffi bindings as well
+/// as rust binaries as python packages
+enum Command {
     #[command(name = "build", alias = "b")]
     /// Build the crate into python packages
     Build {
         /// Build artifacts in release mode, with optimizations
-        #[arg(short = 'r', long)]
+        #[arg(short = 'r', long, help_heading = heading::COMPILATION_OPTIONS, conflicts_with = "profile")]
         release: bool,
-        /// Strip the library for minimum file size
-        #[arg(long)]
-        strip: bool,
+        /// Strip the library for minimum file size.
+        /// Can be set to `true` or `false`, or used as a flag (`--strip` implies `true`).
+        #[arg(
+            long,
+            env = "MATURIN_STRIP",
+            // `--strip` without a value is treated as `--strip true`
+            default_missing_value = "true",
+            num_args = 0..=1,
+            require_equals = false
+        )]
+        strip: Option<bool>,
         /// Build a source distribution
         #[arg(long)]
         sdist: bool,
@@ -52,7 +86,7 @@ enum Opt {
     /// Build and publish the crate as python packages to pypi
     Publish {
         /// Do not pass --release to cargo
-        #[arg(long)]
+        #[arg(long, conflicts_with = "profile")]
         debug: bool,
         /// Do not strip the library for minimum file size
         #[arg(long = "no-strip")]
@@ -69,7 +103,7 @@ enum Opt {
     /// Search and list the available python installations
     ListPython {
         #[arg(long)]
-        target: Option<String>,
+        target: Option<TargetTriple>,
     },
     #[command(name = "develop", alias = "dev")]
     /// Install the crate as module in the current virtualenv
@@ -139,6 +173,10 @@ enum Opt {
     #[cfg(feature = "zig")]
     #[command(subcommand, hide = true)]
     Zig(Zig),
+    /// Generate the JSON schema for the `pyproject.toml` file.
+    #[cfg(feature = "schemars")]
+    #[command(name = "generate-json-schema", hide = true)]
+    GenerateJsonSchema(GenerateJsonSchemaOptions),
 }
 
 /// Backend for the PEP 517 integration. Not for human consumption
@@ -155,20 +193,34 @@ enum Pep517Command {
         /// The metadata_directory argument to prepare_metadata_for_build_wheel
         #[arg(long = "metadata-directory")]
         metadata_directory: PathBuf,
-        /// Strip the library for minimum file size
-        #[arg(long)]
-        strip: bool,
+        /// Strip the library for minimum file size.
+        /// Can be set to `true` or `false`, or used as a flag (`--strip` implies `true`).
+        #[arg(
+            long,
+            env = "MATURIN_STRIP",
+            // `--strip` without a value is treated as `--strip true`
+            default_missing_value = "true",
+            num_args = 0..=1,
+            require_equals = false
+        )]
+        strip: Option<bool>,
     },
     #[command(name = "build-wheel")]
     /// Implementation of build_wheel
-    ///
-    /// --release and --strip are currently unused by the PEP 517 implementation
     BuildWheel {
         #[command(flatten)]
         build_options: BuildOptions,
-        /// Strip the library for minimum file size
-        #[arg(long)]
-        strip: bool,
+        /// Strip the library for minimum file size.
+        /// Can be set to `true` or `false`, or used as a flag (`--strip` implies `true`).
+        #[arg(
+            long,
+            env = "MATURIN_STRIP",
+            // `--strip` without a value is treated as `--strip true`
+            default_missing_value = "true",
+            num_args = 0..=1,
+            require_equals = false
+        )]
+        strip: Option<bool>,
         /// Build editable wheels
         #[arg(long)]
         editable: bool,
@@ -189,6 +241,7 @@ fn detect_venv(target: &Target) -> Result<PathBuf> {
     match (env::var_os("VIRTUAL_ENV"), env::var_os("CONDA_PREFIX")) {
         (Some(dir), None) => return Ok(PathBuf::from(dir)),
         (None, Some(dir)) => return Ok(PathBuf::from(dir)),
+        (Some(venv), Some(conda)) if venv == conda => return Ok(PathBuf::from(venv)),
         (Some(_), Some(_)) => {
             bail!("Both VIRTUAL_ENV and CONDA_PREFIX are set. Please unset one of them")
         }
@@ -242,32 +295,41 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             strip,
         } => {
             assert_eq!(build_options.interpreter.len(), 1);
-            let context = build_options.into_build_context(true, strip, false)?;
+            let mut context = build_options
+                .into_build_context()
+                .strip(strip)
+                .editable(false)
+                .build()?;
 
-            // Since afaik all other PEP 517 backends also return linux tagged wheels, we do so too
-            let tags = match context.bridge() {
-                BridgeModel::Bindings(..) | BridgeModel::Bin(Some(..)) => {
-                    vec![context.interpreter[0].get_tag(&context, &[PlatformTag::Linux])?]
-                }
-                BridgeModel::BindingsAbi3(major, minor) => {
-                    let platform = context.get_platform_tag(&[PlatformTag::Linux])?;
-                    vec![format!("cp{major}{minor}-abi3-{platform}")]
-                }
-                BridgeModel::Bin(None) | BridgeModel::Cffi | BridgeModel::UniFfi => {
-                    context.get_universal_tags(&[PlatformTag::Linux])?.1
-                }
-            };
+            // TBD: does `--profile release` do anything here?
+            if context.cargo_options.profile.is_none() {
+                context.cargo_options.profile = Some("release".to_string());
+            }
 
-            let mut writer = PathWriter::from_path(metadata_directory);
-            write_dist_info(&mut writer, &context.metadata21, &tags)?;
-            println!("{}", context.metadata21.get_dist_info_dir().display());
+            let mut writer =
+                VirtualWriter::new(PathWriter::from_path(metadata_directory), Override::empty());
+            let dist_info_dir = write_dist_info(
+                &mut writer,
+                &context.project_layout.project_root,
+                &context.metadata24,
+                &context.tags_from_bridge()?,
+            )?;
+            writer.finish()?;
+            println!("{}", dist_info_dir.display());
         }
         Pep517Command::BuildWheel {
             build_options,
             strip,
             editable,
         } => {
-            let build_context = build_options.into_build_context(true, strip, editable)?;
+            let mut build_context = build_options
+                .into_build_context()
+                .strip(strip)
+                .editable(editable)
+                .build()?;
+            if build_context.cargo_options.profile.is_none() {
+                build_context.cargo_options.profile = Some("release".to_string());
+            }
             let wheels = build_context.build_wheels()?;
             assert_eq!(wheels.len(), 1);
             println!("{}", wheels[0].0.to_str().unwrap());
@@ -280,11 +342,19 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
                 out: Some(sdist_directory),
                 cargo: CargoOptions {
                     manifest_path,
+                    // Enable all features to ensure all optional path dependencies are packaged
+                    // into source distribution
+                    all_features: true,
                     ..Default::default()
                 },
                 ..Default::default()
             };
-            let build_context = build_options.into_build_context(false, false, false)?;
+            let build_context = build_options
+                .into_build_context()
+                .strip(Some(false))
+                .editable(false)
+                .sdist_only(true)
+                .build()?;
             let (path, _) = build_context
                 .build_source_distribution()?
                 .context("Failed to build source distribution, pyproject.toml not found")?;
@@ -295,10 +365,8 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
     Ok(())
 }
 
+#[instrument]
 fn run() -> Result<()> {
-    #[cfg(feature = "log")]
-    tracing_subscriber::fmt::init();
-
     #[cfg(feature = "zig")]
     {
         // Allow symlink `maturin` to `ar` to invoke `zig ar`
@@ -320,14 +388,24 @@ fn run() -> Result<()> {
     #[cfg(feature = "wild")]
     let opt = Opt::parse_from(wild::args_os());
 
-    match opt {
-        Opt::Build {
-            build,
+    setup_logging(opt.verbose)?;
+
+    match opt.command {
+        Command::Build {
+            mut build,
             release,
             strip,
             sdist,
         } => {
-            let build_context = build.into_build_context(release, strip, false)?;
+            // set profile to release if specified; `--release` and `--profile` are mutually exclusive
+            if release {
+                build.profile = Some("release".to_string());
+            }
+            let build_context = build
+                .into_build_context()
+                .strip(strip)
+                .editable(false)
+                .build()?;
             if sdist {
                 build_context
                     .build_source_distribution()?
@@ -337,34 +415,52 @@ fn run() -> Result<()> {
             assert!(!wheels.is_empty());
         }
         #[cfg(feature = "upload")]
-        Opt::Publish {
-            build,
-            publish,
+        Command::Publish {
+            mut build,
+            mut publish,
             debug,
             no_strip,
             no_sdist,
         } => {
-            let build_context = build.into_build_context(!debug, !no_strip, false)?;
+            // set profile to dev if specified; `--debug` and `--profile` are mutually exclusive
+            //
+            // do it here to take precedence over pyproject.toml profile setting
+            if debug {
+                build.profile = Some("dev".to_string());
+            }
 
-            if !build_context.release {
+            let mut build_context = build
+                .into_build_context()
+                .strip(Some(!no_strip))
+                .editable(false)
+                .build()?;
+
+            // ensure profile always set when publishing
+            // (respect pyproject.toml if set)
+            // don't need to check `debug` here, set above to take precedence if set
+            let profile = build_context
+                .cargo_options
+                .profile
+                .get_or_insert_with(|| "release".to_string());
+
+            if profile == "dev" {
                 eprintln!("⚠️  Warning: You're publishing debug wheels");
             }
 
             let mut wheels = build_context.build_wheels()?;
-            if !no_sdist {
-                if let Some(sd) = build_context.build_source_distribution()? {
-                    wheels.push(sd);
-                }
+            if !no_sdist && let Some(sd) = build_context.build_source_distribution()? {
+                wheels.push(sd);
             }
 
             let items = wheels.into_iter().map(|wheel| wheel.0).collect::<Vec<_>>();
+            publish.non_interactive_on_ci();
 
             upload_ui(&items, &publish)?
         }
-        Opt::ListPython { target } => {
+        Command::ListPython { target } => {
             let found = if target.is_some() {
-                let target = Target::from_target_triple(target)?;
-                PythonInterpreter::find_by_target(&target, None)
+                let target = Target::from_target_triple(target.as_ref())?;
+                PythonInterpreter::find_by_target(&target, None, None)
             } else {
                 let target = Target::from_target_triple(None)?;
                 // We don't know the targeted bindings yet, so we use the most lenient
@@ -375,51 +471,78 @@ fn run() -> Result<()> {
                 eprintln!(" - {interpreter}");
             }
         }
-        Opt::Develop(develop_options) => {
-            let target = Target::from_target_triple(develop_options.cargo_options.target.clone())?;
+        Command::Develop(develop_options) => {
+            let target = Target::from_target_triple(develop_options.cargo_options.target.as_ref())?;
             let venv_dir = detect_venv(&target)?;
             develop(develop_options, &venv_dir)?;
         }
-        Opt::SDist { manifest_path, out } => {
+        Command::SDist { manifest_path, out } => {
+            // Get cargo metadata to check for path dependencies
+            let cargo_metadata_result = cargo_metadata::MetadataCommand::new()
+                .cargo_path("cargo")
+                .manifest_path(
+                    manifest_path
+                        .as_deref()
+                        .unwrap_or_else(|| std::path::Path::new("Cargo.toml")),
+                )
+                .verbose(true)
+                .exec();
+
+            let has_path_deps = cargo_metadata_result
+                .ok()
+                .and_then(|metadata| find_path_deps(&metadata).ok())
+                .map(|path_deps| !path_deps.is_empty())
+                .unwrap_or(false); // If we can't get metadata, don't force all features
             let build_options = BuildOptions {
                 out,
                 cargo: CargoOptions {
                     manifest_path,
+                    // Only enable all features when we have local path dependencies
+                    // to ensure they are packaged into source distribution
+                    all_features: has_path_deps,
                     ..Default::default()
                 },
                 ..Default::default()
             };
-            let build_context = build_options.into_build_context(false, false, false)?;
+            let build_context = build_options
+                .into_build_context()
+                .strip(Some(false))
+                .editable(false)
+                .sdist_only(true)
+                .build()?;
             build_context
                 .build_source_distribution()?
                 .context("Failed to build source distribution, pyproject.toml not found")?;
         }
-        Opt::Pep517(subcommand) => pep517(subcommand)?,
+        Command::Pep517(subcommand) => pep517(subcommand)?,
         #[cfg(feature = "scaffolding")]
-        Opt::InitProject { path, options } => init_project(path, options)?,
+        Command::InitProject { path, options } => init_project(path, options)?,
         #[cfg(feature = "scaffolding")]
-        Opt::NewProject { path, options } => new_project(path, options)?,
+        Command::NewProject { path, options } => new_project(path, options)?,
         #[cfg(feature = "scaffolding")]
-        Opt::GenerateCI(generate_ci) => generate_ci.execute()?,
+        Command::GenerateCI(generate_ci) => generate_ci.execute()?,
         #[cfg(feature = "upload")]
-        Opt::Upload { publish, files } => {
+        Command::Upload { mut publish, files } => {
             if files.is_empty() {
                 eprintln!("⚠️  Warning: No files given, exiting.");
                 return Ok(());
             }
+            publish.non_interactive_on_ci();
 
             upload_ui(&files, &publish)?
         }
         #[cfg(feature = "cli-completion")]
-        Opt::Completions { shell } => {
+        Command::Completions { shell } => {
             shell.generate(&mut Opt::command(), &mut std::io::stdout());
         }
         #[cfg(feature = "zig")]
-        Opt::Zig(subcommand) => {
+        Command::Zig(subcommand) => {
             subcommand
                 .execute()
                 .context("Failed to run zig linker wrapper")?;
         }
+        #[cfg(feature = "schemars")]
+        Command::GenerateJsonSchema(args) => generate_json_schema(args)?,
     }
 
     Ok(())
@@ -447,6 +570,33 @@ fn setup_panic_hook() {
     }));
 }
 
+fn setup_logging(verbose: u8) -> Result<()> {
+    // `RUST_LOG` takes precedence over these
+    let default_directive = match verbose {
+        // `-v` runs `cargo build -v`, but doesn't show maturin debug logging yet.
+        0..=1 => tracing::level_filters::LevelFilter::OFF.into(),
+        2 => Directive::from_str("debug").unwrap(),
+        3.. => Directive::from_str("trace").unwrap(),
+    };
+
+    let filter = EnvFilter::builder()
+        .with_default_directive(default_directive)
+        .from_env()
+        .context("Invalid RUST_LOG directives")?;
+
+    let logger = tracing_subscriber::fmt::layer()
+        // Avoid showing all the details from the spans
+        .compact()
+        // Log the timing of each span
+        .with_span_events(tracing_subscriber::fmt::format::FmtSpan::CLOSE);
+
+    tracing_subscriber::registry()
+        .with(logger.with_filter(filter))
+        .init();
+
+    Ok(())
+}
+
 fn main() {
     #[cfg(not(debug_assertions))]
     setup_panic_hook();
