@@ -1,7 +1,7 @@
 //! A pyproject.toml as specified in PEP 517
 
-use crate::auditwheel::AuditWheelMode;
 use crate::PlatformTag;
+use crate::auditwheel::AuditWheelMode;
 use anyhow::{Context, Result};
 use fs_err as fs;
 use pep440_rs::Version;
@@ -80,7 +80,7 @@ impl GlobPattern {
     pub fn targets(&self, format: Format) -> Option<&str> {
         match self {
             // Not specified defaults to both
-            Self::Path(ref glob) => Some(glob),
+            Self::Path(glob) => Some(glob),
             Self::WithFormat {
                 path,
                 format: formats,
@@ -163,6 +163,24 @@ pub enum SdistGenerator {
     Git,
 }
 
+/// SBOM configuration
+#[derive(Serialize, Deserialize, Debug, Clone, Default, PartialEq, Eq)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
+pub struct SbomConfig {
+    /// Generate an SBOM for Rust crates. Defaults to `true`.
+    pub rust: Option<bool>,
+    /// Generate a CycloneDX SBOM for external shared libraries grafted during
+    /// auditwheel repair. Defaults to `true` when repair copies libraries.
+    ///
+    /// The SBOM is written to `<dist-info>/sboms/auditwheel.cdx.json` and
+    /// records which OS packages (deb, rpm, apk) provided the grafted
+    /// libraries, following the same convention as Python's auditwheel.
+    pub auditwheel: Option<bool>,
+    /// Additional SBOM files to include in the `.dist-info/sboms` directory.
+    pub include: Option<Vec<PathBuf>>,
+}
+
 /// The `[tool.maturin]` section of a pyproject.toml
 #[derive(Serialize, Deserialize, Debug, Clone, Default)]
 #[serde(rename_all = "kebab-case")]
@@ -232,6 +250,8 @@ pub struct ToolMaturin {
     // in venv.
     #[serde(default)]
     pub use_base_python: bool,
+    /// SBOM configuration
+    pub sbom: Option<SbomConfig>,
 }
 
 /// A pyproject.toml as specified in PEP 517
@@ -248,6 +268,8 @@ pub struct PyProjectToml {
     ///
     /// We use it for `[tool.maturin]`
     pub tool: Option<Tool>,
+    /// PEP 735: Dependency groups
+    pub dependency_groups: Option<pyproject_toml::DependencyGroups>,
 }
 
 impl PyProjectToml {
@@ -447,11 +469,15 @@ impl PyProjectToml {
             .as_ref()
             .is_some_and(|d| d.iter().any(|s| s == "version"));
         if has_static_version && has_dynamic_version {
-            eprintln!("⚠️  Warning: `project.dynamic` must not specify `version` when `project.version` is present in pyproject.toml");
+            eprintln!(
+                "⚠️  Warning: `project.dynamic` must not specify `version` when `project.version` is present in pyproject.toml"
+            );
             return false;
         }
         if !has_static_version && !has_dynamic_version {
-            eprintln!("⚠️  Warning: `project.version` field is required in pyproject.toml unless it is present in the `project.dynamic` list");
+            eprintln!(
+                "⚠️  Warning: `project.version` field is required in pyproject.toml unless it is present in the `project.dynamic` list"
+            );
             return false;
         }
         true
@@ -461,8 +487,8 @@ impl PyProjectToml {
 #[cfg(test)]
 mod tests {
     use crate::{
-        pyproject_toml::{Format, Formats, GlobPattern, ToolMaturin},
         PyProjectToml,
+        pyproject_toml::{Format, Formats, GlobPattern, ToolMaturin},
     };
     use expect_test::expect;
     use fs_err as fs;
