@@ -1,6 +1,8 @@
-use crate::common::{check_installed, create_conda_env, create_virtualenv, maybe_mock_cargo};
+use crate::common::{
+    TestInstallBackend, check_installed, create_conda_env, create_virtualenv, maybe_mock_cargo,
+};
 use anyhow::Result;
-use maturin::{develop, CargoOptions, DevelopOptions};
+use maturin::{CargoOptions, DevelopOptions, develop};
 use std::path::{Path, PathBuf};
 use std::process::Command;
 use std::str;
@@ -12,6 +14,7 @@ pub fn test_develop(
     bindings: Option<String>,
     unique_name: &str,
     conda: bool,
+    test_backend: TestInstallBackend,
 ) -> Result<()> {
     maybe_mock_cargo();
 
@@ -25,22 +28,32 @@ pub fn test_develop(
     // Ensure the test doesn't wrongly pass
     check_installed(package, &python).unwrap_err();
 
-    let output = Command::new(&python)
-        .args([
-            "-m",
-            "pip",
-            "install",
-            "--disable-pip-version-check",
-            "cffi",
-        ])
-        .output()?;
-    if !output.status.success() {
-        panic!(
-            "Failed to install cffi: {}\n---stdout:\n{}---stderr:\n{}",
-            output.status,
-            str::from_utf8(&output.stdout)?,
-            str::from_utf8(&output.stderr)?
-        );
+    let uv = matches!(test_backend, TestInstallBackend::Uv);
+    let mut pip_packages = Vec::new();
+    if unique_name.contains("cffi") {
+        pip_packages.push("cffi");
+    }
+    if cfg!(any(
+        target_os = "linux",
+        target_os = "macos",
+        target_os = "windows"
+    )) && uv
+    {
+        pip_packages.push("uv");
+    }
+    if !pip_packages.is_empty() {
+        let mut cmd = Command::new(&python);
+        cmd.args(["-m", "pip", "install", "--disable-pip-version-check"])
+            .args(pip_packages);
+        let output = cmd.output()?;
+        if !output.status.success() {
+            panic!(
+                "Failed to install cffi: {}\n---stdout:\n{}---stderr:\n{}",
+                output.status,
+                str::from_utf8(&output.stdout)?,
+                str::from_utf8(&output.stderr)?
+            );
+        }
     }
 
     let manifest_file = package.join("Cargo.toml");
@@ -49,13 +62,17 @@ pub fn test_develop(
         release: false,
         strip: false,
         extras: Vec::new(),
+        group: Vec::new(),
         skip_install: false,
+        pip_path: None,
         cargo_options: CargoOptions {
             manifest_path: Some(manifest_file),
             quiet: true,
             target_dir: Some(PathBuf::from(format!("test-crates/targets/{unique_name}"))),
             ..Default::default()
         },
+        uv,
+        compression: Default::default(),
     };
     develop(develop_options, &venv_dir)?;
 
