@@ -15,6 +15,100 @@ use std::time::Duration;
 use time::macros::datetime;
 use which::which;
 
+#[test]
+fn sdist_excludes_default_run() {
+    let temp_dir = tempfile::tempdir().unwrap();
+    let project_dir = temp_dir.path().join("hello-world");
+    common::other::copy_dir_recursive(Path::new("test-crates/hello-world"), &project_dir).unwrap();
+
+    // Add excluded bin and set as default-run
+    let cargo_toml_path = project_dir.join("Cargo.toml");
+    let mut cargo_toml = fs_err::read_to_string(&cargo_toml_path).unwrap();
+
+    // Create two different README files: one at the project root (for pyproject.toml)
+    // and one at a parent directory (for Cargo.toml). Both resolve to "README.md" in the
+    // sdist, reproducing https://github.com/PyO3/maturin/issues/2358.
+    fs_err::write(project_dir.join("README.md"), "Pyproject readme").unwrap();
+    let parent_readme = temp_dir.path().join("README.md");
+    fs_err::write(&parent_readme, "Cargo readme").unwrap();
+    cargo_toml = cargo_toml.replace("../../README.md", "../README.md");
+
+    cargo_toml.push_str("\n[[bin]]\nname = \"excluded_bin\"\npath = \"src/bin/excluded_bin.rs\"\n");
+    // Replace default-run
+    let cargo_toml = cargo_toml.replace(
+        "default-run = \"hello-world\"",
+        "default-run = \"excluded_bin\"",
+    );
+    fs_err::write(&cargo_toml_path, cargo_toml).unwrap();
+
+    // Add source file
+    let bin_path = project_dir.join("src/bin/excluded_bin.rs");
+    // Ensure parent dir exists (src/bin might be missing if I just copied structure and it was empty? No, hello-world has src/bin/foo.rs)
+    fs_err::write(&bin_path, "fn main() {}").unwrap();
+
+    // Add exclusion and set a pyproject.toml readme that differs from Cargo.toml's readme.
+    // This reproduces https://github.com/PyO3/maturin/issues/2358 where having readme in
+    // both Cargo.toml and pyproject.toml (pointing to different files) caused a
+    // "was already added" error.
+    let pyproject_toml_path = project_dir.join("pyproject.toml");
+    let mut pyproject_toml = fs_err::read_to_string(&pyproject_toml_path).unwrap();
+    pyproject_toml =
+        pyproject_toml.replace("exclude = [", "exclude = [\n  \"src/bin/excluded_bin.rs\",");
+    // Set pyproject.toml readme = "README.md" so both Cargo.toml and pyproject.toml
+    // add a file at the same target path (README.md) from different sources.
+    // The Cargo.toml README.md was already written above with different content.
+    pyproject_toml = pyproject_toml.replace(
+        "dynamic = [\"authors\", \"readme\"]",
+        "readme = \"README.md\"\ndynamic = [\"authors\"]",
+    );
+    fs_err::write(&pyproject_toml_path, pyproject_toml).unwrap();
+
+    // Expect Cargo.toml NOT to have default-run="excluded_bin"
+    let expected_cargo_toml = expect![[r#"
+        [package]
+        name = "hello-world"
+        version = "0.1.0"
+        authors = ["konstin <konstin@mailbox.org>"]
+        edition = "2021"
+        # Test references to out-of-project files
+        readme = "README.md"
+
+        [dependencies]
+
+        [[bench]]
+        name = "included_bench"
+
+        [[example]]
+        name = "included_example"
+    "#]];
+
+    handle_result(other::test_source_distribution(
+        &project_dir,
+        SdistGenerator::Cargo,
+        expect![[r#"
+            {
+                "hello_world-0.1.0/Cargo.lock",
+                "hello_world-0.1.0/Cargo.toml",
+                "hello_world-0.1.0/LICENSE",
+                "hello_world-0.1.0/PKG-INFO",
+                "hello_world-0.1.0/README.md",
+                "hello_world-0.1.0/benches/included_bench.rs",
+                "hello_world-0.1.0/check_installed/check_installed.py",
+                "hello_world-0.1.0/examples/included_example.rs",
+                "hello_world-0.1.0/licenses/AUTHORS.txt",
+                "hello_world-0.1.0/pyproject.toml",
+                "hello_world-0.1.0/src/bin/foo.rs",
+                "hello_world-0.1.0/src/main.rs",
+            }
+        "#]],
+        Some((
+            Path::new("hello_world-0.1.0/Cargo.toml"),
+            expected_cargo_toml,
+        )),
+        "sdist-hello-world-default-run",
+    ))
+}
+
 mod common;
 
 #[test]
@@ -227,6 +321,19 @@ fn develop_hello_world(#[case] backend: TestInstallBackend, #[case] name: &str)
     ));
 }
 
+/// Test editable install of a project with both a binary and a Python module.
+/// This is a regression test for https://github.com/PyO3/maturin/issues/2933
+#[test]
+fn develop_bin_with_python_module() {
+    handle_result(develop::test_develop(
+        "test-crates/bin-with-python-module",
+        None,
+        "develop-bin-with-python-module",
+        false,
+        TestInstallBackend::Pip,
+    ));
+}
+
 #[rstest]
 #[timeout(Duration::from_secs(120))]
 #[case(TestInstallBackend::Pip, "pip")]
@@ -580,8 +687,8 @@ fn invalid_manylinux_does_not_panic() {
 }
 
 #[test]
-fn warn_on_missing_python_source() {
-    handle_result(errors::warn_on_missing_python_source())
+fn error_on_missing_python_source() {
+    handle_result(errors::error_on_missing_python_source())
 }
 
 #[test]
@@ -628,6 +735,7 @@ fn workspace_members_beneath_pyproject_sdist() {
         expect![[r#"
             {
                 "pyo3_mixed_workspace-2.1.3/PKG-INFO",
+                "pyo3_mixed_workspace-2.1.3/README.md",
                 "pyo3_mixed_workspace-2.1.3/pyproject.toml",
                 "pyo3_mixed_workspace-2.1.3/rust/Cargo.lock",
                 "pyo3_mixed_workspace-2.1.3/rust/Cargo.toml",
@@ -759,6 +867,48 @@ fn lib_with_target_path_dep_sdist() {
     ))
 }
 
+/// Regression test for https://github.com/PyO3/maturin/issues/2202
+/// When `python-source` points outside the Rust source directory,
+/// `maturin sdist` used to panic with a `StripPrefixError`.
+#[test]
+fn external_python_source_sdist() {
+    let pyproject_toml = expect![[r#"
+        [project]
+        name = "external-python-source"
+        version = "0.1.0"
+
+        [build-system]
+        requires = ["maturin>=1.0,<2.0"]
+        build-backend = "maturin"
+
+        [tool.maturin]
+        bindings = "bin"
+        module-name = "external_python_source"
+        manifest-path = "crate/Cargo.toml"
+        python-source = "python"
+    "#]];
+    handle_result(other::test_source_distribution(
+        "test-crates/external-python-source/crate",
+        SdistGenerator::Cargo,
+        expect![[r#"
+            {
+                "external_python_source-0.1.0/PKG-INFO",
+                "external_python_source-0.1.0/crate/.gitignore",
+                "external_python_source-0.1.0/crate/Cargo.lock",
+                "external_python_source-0.1.0/crate/Cargo.toml",
+                "external_python_source-0.1.0/crate/src/main.rs",
+                "external_python_source-0.1.0/pyproject.toml",
+                "external_python_source-0.1.0/python/external_python_source/__init__.py",
+            }
+        "#]],
+        Some((
+            Path::new("external_python_source-0.1.0/pyproject.toml"),
+            pyproject_toml,
+        )),
+        "sdist-external-python-source",
+    ))
+}
+
 #[test]
 fn pyo3_mixed_src_layout_sdist() {
     handle_result(other::test_source_distribution(
@@ -767,6 +917,7 @@ fn pyo3_mixed_src_layout_sdist() {
         expect![[r#"
             {
                 "pyo3_mixed_src-2.1.3/PKG-INFO",
+                "pyo3_mixed_src-2.1.3/README.md",
                 "pyo3_mixed_src-2.1.3/pyproject.toml",
                 "pyo3_mixed_src-2.1.3/rust/Cargo.lock",
                 "pyo3_mixed_src-2.1.3/rust/Cargo.toml",
@@ -839,21 +990,70 @@ fn pyo3_mixed_include_exclude_git_sdist_generator() {
     ))
 }
 
+#[test]
+#[cfg(feature = "sbom")]
+fn pyo3_pure_sbom_wheel_files() {
+    handle_result(other::check_wheel_files_with_sbom(
+        "test-crates/pyo3-pure",
+        "wheel-files-pyo3-pure-sbom",
+    ))
+}
+
 #[test]
 fn pyo3_mixed_include_exclude_wheel_files() {
+    #[allow(unused_mut)]
+    let mut expected = vec![
+        "pyo3_mixed_include_exclude-2.1.3.dist-info/METADATA",
+        "pyo3_mixed_include_exclude-2.1.3.dist-info/RECORD",
+        "pyo3_mixed_include_exclude-2.1.3.dist-info/WHEEL",
+        "pyo3_mixed_include_exclude-2.1.3.dist-info/entry_points.txt",
+        "pyo3_mixed_include_exclude/__init__.py",
+        "pyo3_mixed_include_exclude/include_this_file",
+        "pyo3_mixed_include_exclude/python_module/__init__.py",
+        "pyo3_mixed_include_exclude/python_module/double.py",
+        "README.md",
+    ];
+    #[cfg(feature = "sbom")]
+    expected.push("pyo3_mixed_include_exclude-2.1.3.dist-info/sboms/pyo3-mixed-include-exclude.cyclonedx.json");
     handle_result(other::check_wheel_files(
         "test-crates/pyo3-mixed-include-exclude",
-        vec![
-            "pyo3_mixed_include_exclude-2.1.3.dist-info/METADATA",
-            "pyo3_mixed_include_exclude-2.1.3.dist-info/RECORD",
-            "pyo3_mixed_include_exclude-2.1.3.dist-info/WHEEL",
-            "pyo3_mixed_include_exclude-2.1.3.dist-info/entry_points.txt",
-            "pyo3_mixed_include_exclude/__init__.py",
-            "pyo3_mixed_include_exclude/include_this_file",
-            "pyo3_mixed_include_exclude/python_module/__init__.py",
-            "pyo3_mixed_include_exclude/python_module/double.py",
-            "README.md",
-        ],
+        expected,
+        "wheel-files-pyo3-mixed-include-exclude",
+    ))
+}
+
+// Regression test for https://github.com/PyO3/maturin/issues/2691
+// Verifies that `include` patterns work correctly with `python-source` (src-layout),
+// where include paths should be resolved relative to the project root, not the python dir.
+#[test]
+fn pyo3_mixed_py_subdir_include_wheel_files() {
+    #[allow(unused_mut)]
+    let mut expected = vec![
+        "pyo3_mixed_py_subdir-2.1.3.dist-info/METADATA",
+        "pyo3_mixed_py_subdir-2.1.3.dist-info/RECORD",
+        "pyo3_mixed_py_subdir-2.1.3.dist-info/WHEEL",
+        "pyo3_mixed_py_subdir-2.1.3.dist-info/entry_points.txt",
+        "pyo3_mixed_py_subdir/__init__.py",
+        "pyo3_mixed_py_subdir/python_module/__init__.py",
+        "pyo3_mixed_py_subdir/python_module/double.py",
+        "assets/extra_data.txt",
+    ];
+    #[cfg(feature = "sbom")]
+    expected.push("pyo3_mixed_py_subdir-2.1.3.dist-info/sboms/pyo3-mixed-py-subdir.cyclonedx.json");
+    handle_result(other::check_wheel_files(
+        "test-crates/pyo3-mixed-py-subdir",
+        expected,
+        "wheel-files-pyo3-mixed-py-subdir-include",
+    ))
+}
+
+// Tests that paths in the wheel `RECORD` use `/` instead of `\\`. Even
+// (especially, exclusively) on Windows.
+#[test]
+fn pyo3_wheel_record_has_normalized_paths() {
+    handle_result(other::check_wheel_paths(
+        "test-crates/pyo3-mixed-include-exclude",
+        "pyo3_mixed_include_exclude-2.1.3.dist-info/RECORD",
         "wheel-files-pyo3-mixed-include-exclude",
     ))
 }
@@ -948,6 +1148,24 @@ fn workspace_inheritance_sdist() {
 
 #[test]
 fn workspace_license_files() {
+    let cargo_toml = expect![[r#"
+        [package]
+        name = "hello-world"
+        version = "0.1.0"
+        authors = ["konstin <konstin@mailbox.org>"]
+        edition = "2021"
+        # Test references to out-of-project files
+        readme = "README.md"
+        default-run = "hello-world"
+
+        [dependencies]
+
+        [[bench]]
+        name = "included_bench"
+
+        [[example]]
+        name = "included_example"
+    "#]];
     handle_result(other::test_source_distribution(
         "test-crates/hello-world",
         SdistGenerator::Cargo,
@@ -958,14 +1176,16 @@ fn workspace_license_files() {
                 "hello_world-0.1.0/LICENSE",
                 "hello_world-0.1.0/PKG-INFO",
                 "hello_world-0.1.0/README.md",
+                "hello_world-0.1.0/benches/included_bench.rs",
                 "hello_world-0.1.0/check_installed/check_installed.py",
+                "hello_world-0.1.0/examples/included_example.rs",
                 "hello_world-0.1.0/licenses/AUTHORS.txt",
                 "hello_world-0.1.0/pyproject.toml",
                 "hello_world-0.1.0/src/bin/foo.rs",
                 "hello_world-0.1.0/src/main.rs",
             }
         "#]],
-        None,
+        Some((Path::new("hello_world-0.1.0/Cargo.toml"), cargo_toml)),
         "sdist-hello-world",
     ))
 }
